//-----------------------------------------------------------------------------
// MK3_2560_Terminal
// Version 1.4
// 05.06.2012 / UB
// "www.Mikrocontroller-4U.de"
// mc-4u(@)t-online.de
//-----------------------------------------------------------------------------
// Fuer das myAVR MK3 Board
// mit ATMega-2560 @ 16MHz
// Terminal-Programm
// um per PC ueber RS232 auf die Port-Pins der CPU
// zugreifen zu koennen
//-----------------------------------------------------------------------------
// das File muss im Ordner
// "myAVR/WinAVR/AVR/Include/UB_MK3_2560/"
// liegen und mit #include <UB_MK3_2560/MK3_2560_Terminal.h>
// eingebunden werden
//-----------------------------------------------------------------------------
// Belegt : nichts (auser einem COM-Port)
// Benutzt : V24-LIB : MK3_2560_V24
//           event. ADC-LIB : MK3_2560_ADC
//
// Hinweis :
//
// > der DEFINE-Schalter "TERMINAL_HW_MODE"
//   muss zur Hardware passen !!
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Hier koennen Funktionen die nicht benutzt werden
// abgeschaltet werden...einfach den DEFINE von "1" auf "0" aendern
//-----------------------------------------------------------------------------

#define USE_MK3_TERMINAL_DDR 1   // zum lesen/setzen vom DD-Register
#define USE_MK3_TERMINAL_PORT 1  // zum lesen/setzen eines PORTs
#define USE_MK3_TERMINAL_BIT 1   // zum lesen/setzen eines Bits
#define USE_MK3_TERMINAL_ADC 1   // zum lesen vom ADC


//-----------------------------------------------------------------------------
// Hardware auswahl
//-----------------------------------------------------------------------------
#define TERMINAL_HW_MODE    0       // 0=MK3-Board, 1=MK1/MK2-Board/mySmartControl


//----------------------------------------------------------------------------
// Includes
//----------------------------------------------------------------------------
#include "ub_mk3_2560/MK3_2560_V24.h" //  bindet die V24-Funktionen ein
#include <stdio.h>
#include <string.h>

#if USE_MK3_TERMINAL_ADC==1
#include "ub_mk3_2560/MK3_2560_ADC.h" //  bindet die ADC-Funktionen ein
#endif



//-----------------------------------------------------------------------------
// INFOS zur Benutzung :
// 01. Funktion "MK3_INIT_TERMINAL" starten
// 02. Funktion "MK3_TERMINAL_LOOP" muss in der While(1) aufgerufen werden
// 03. per RS232 koennen jetzt Kommandos an das MK3 gesendet werden
// 04. das Kommando "HELP" listet alle Kommandos auf
//-----------------------------------------------------------------------------



//-----------------------------------------------------------------------------
// Enumerationen fuer die Funktioen
//-----------------------------------------------------------------------------
typedef enum {
  NO_CMD = 0, // kein Kommando empfangen
  CMD_OK,     // Kommando empfangen und bearbeitet
  CMD_NC      // kommando nicht verstanden (Daten stehen in Terminal.rx[])
}CMD_TYP;


//-----------------------------------------------------------------------------
// Funktionen die vom MAIN benutzt werden koennen
//-----------------------------------------------------------------------------
void MK3_INIT_TERMINAL(COMPORT port, BAUDRATE baud);  // zum init des Terminal
CMD_TYP MK3_TERMINAL_LOOP(void);                      // Abfrage auf Daten
//-----------------------------------------------------------------------------


#define TX_PUFFER_LEN 30    // laenge des Sende-Puffers

//-----------------------------------------------------------------------------
// Globale Variablen
//-----------------------------------------------------------------------------
typedef struct {
  char rx[RX_PUFFER_LEN+1];  // RS232 RX-Puffer
  char tx[TX_PUFFER_LEN+1];  // RS232 TX-Puffer
  COMPORT com;
}Terminal_t;
Terminal_t Terminal;



//-----------------------------------------------------------------------------
// interne Funktionen
//-----------------------------------------------------------------------------
CMD_TYP chk_terminal_CMD(void);
int hex2int(char *ptr,unsigned int start,char laenge);
int hex2ziffer(char ziffer);



//-----------------------------------------------------------------------------
// Initialisiert das Terminal
//-----------------------------------------------------------------------------
void MK3_INIT_TERMINAL(COMPORT port, BAUDRATE baud) {
  // Init der V24-Schnittstelle
  Terminal.com=port;
  MK3_INIT_V24(port,baud);

  // init vom ADC
  #if USE_MK3_TERMINAL_ADC==1
  MK3_INIT_ADC();
  #endif

  // ausgabe vom Terminal initialisierung Screen
  MK3_V24_OUT(Terminal.com," ",RX_END_CRLF);
  MK3_V24_OUT(Terminal.com,"MK3-TERMINAL",RX_END_CRLF);
  MK3_V24_OUT(Terminal.com,"V:1.4 / 05.06.2012 / UB",RX_END_CRLF);
  #if TERMINAL_HW_MODE==0
    MK3_V24_OUT(Terminal.com,"Mode=MK3, ATMega2560",RX_END_CRLF);
  #else
    MK3_V24_OUT(Terminal.com,"Mode=MK2, ATMega8",RX_END_CRLF);
  #endif
  MK3_V24_OUT(Terminal.com,"send HELP for Info",RX_END_CRLF);
}


//-----------------------------------------------------------------------------
// Hauptschleife vom Terminal
// hier wird geprueft ob etwas empfangen wurde
// und wenn was empfangen wurde wird es ausgewertet
// 
// rueckgabewert : NO_CMD=keine Daten empfangen
//                 CMD_OK=guelige Daten empfangen und bearbeitet
//                 CMD_NC=unbekannte Daten empfangen (Daten stehen in Terminal.rx[])
//-----------------------------------------------------------------------------
CMD_TYP MK3_TERMINAL_LOOP(void) {
  CMD_TYP ret_wert=NO_CMD;

  // V24-Schnittstelle pollen
  if(MK3_V24_IN(Terminal.com,Terminal.rx)) {
    // wenn Daten empfangen wurden, pruefen
    ret_wert=chk_terminal_CMD();
  }

  return(ret_wert);
}

//-----------------------------------------------------------------------------
// auswerten des empfangenen Kommandos
//
// rueckgabewert : CMD_OK=guelige Daten empfangen und bearbeitet
//                 CMD_NC=unbekannte Daten empfangen (Daten stehen in Terminal.rx[])
//-----------------------------------------------------------------------------
CMD_TYP chk_terminal_CMD(void) {
  CMD_TYP ret_wert=CMD_OK;
  unsigned char wert=0;
  char buf[7];
  char mode=99;
  char kanal=0;
  int int_wert=0;
  unsigned int uint_wert=0;

  //----------------------------------
  // pruefen des empfangenen Kommandos
  // RX = "HELP"
  // TX = Alle implementierten Kommandos
  //----------------------------------
  if(strncmp(Terminal.rx,"HELP",4)==0) {
    // HELP listet alle Kommandos auf
    mode=0;
    MK3_V24_OUT(Terminal.com," ",RX_END_CRLF);
    MK3_V24_OUT(Terminal.com,"TERMINAL_HELP",RX_END_CRLF);
    MK3_V24_OUT(Terminal.com,"CMD_LIST=",RX_END_CRLF);
    #if USE_MK3_TERMINAL_DDR==1
    MK3_V24_OUT(Terminal.com,"RD_DDRx -> read DD-Register [RD_DDRC]",RX_END_CRLF);
    MK3_V24_OUT(Terminal.com,"WR_DDRx=dd -> write DD-Register [WR_DDRC=F0]",RX_END_CRLF);
    #endif
    #if USE_MK3_TERMINAL_PORT==1
    MK3_V24_OUT(Terminal.com,"RD_PORTx -> read PORT [RD_PORTC]",RX_END_CRLF);
    MK3_V24_OUT(Terminal.com,"WR_PORTx=dd -> write PORT [WR_PORTC=F0]",RX_END_CRLF);
    #endif
    #if USE_MK3_TERMINAL_BIT==1
    MK3_V24_OUT(Terminal.com,"RD_BITxy -> read BIT [RD_BITC3]",RX_END_CRLF);
    MK3_V24_OUT(Terminal.com,"WR_BITxy=d -> write BIT [WR_BITC3=1]",RX_END_CRLF);
    #endif
    #if USE_MK3_TERMINAL_ADC==1
    MK3_V24_OUT(Terminal.com,"RD_ADCy -> read ADC [RD_ADC3]",RX_END_CRLF);
    #endif
    MK3_V24_OUT(Terminal.com,"LIST_END",RX_END_CRLF);
  }
  //----------------------------------
  // DD-Register auslesen
  // RX = "RD_DDRx"  (x=A,B,c...L)
  // TX = "RD_DDRx=ww" (ww=wert in Hex)
  //----------------------------------
  #if USE_MK3_TERMINAL_DDR==1
  else if(strncmp(Terminal.rx,"RD_DDR",6)==0) {
    if(strlen(Terminal.rx)==7) {
      mode=1;
      #if TERMINAL_HW_MODE==0
        // Hardware = myAVR MK3-Board
        if(Terminal.rx[6]=='A') wert=DDRA;
        else if(Terminal.rx[6]=='B') wert=DDRB;
        else if(Terminal.rx[6]=='C') wert=DDRC;
        else if(Terminal.rx[6]=='D') wert=DDRD;
        else if(Terminal.rx[6]=='E') wert=DDRE;
        else if(Terminal.rx[6]=='F') wert=DDRF;
        else if(Terminal.rx[6]=='G') wert=DDRG;
        else if(Terminal.rx[6]=='H') wert=DDRH;
        else if(Terminal.rx[6]=='J') wert=DDRJ;
        else if(Terminal.rx[6]=='K') wert=DDRK;
        else if(Terminal.rx[6]=='L') wert=DDRL;
        else mode=99;
      #else
        // Hardware = myAVR MK1/MK2-Board + mySmartControl
        if(Terminal.rx[6]=='B') wert=DDRB;
        else if(Terminal.rx[6]=='C') wert=DDRC;
        else if(Terminal.rx[6]=='D') wert=DDRD;
        else mode=99;
      #endif
      if(mode==1) {
        strcpy(Terminal.tx,"RD_DDR");
        Terminal.tx[6]=Terminal.rx[6];
        Terminal.tx[7]='=';
        Terminal.tx[8]=0x00;
      }
    }
  } 
  //----------------------------------
  // DD-Register setzen
  // RX = "WR_DDRx=ww"  (x=A,B,c...L / ww=Wert in Hex)
  // TX = "WR_DDRx=ww_OK" (ww=wert in Hex)
  //----------------------------------
  else if(strncmp(Terminal.rx,"WR_DDR",6)==0) {
    if(strlen(Terminal.rx)==10) {
      mode=2;
      #if TERMINAL_HW_MODE==0
        // Hardware = myAVR MK3-Board
        if(Terminal.rx[6]=='A') kanal=0;
        else if(Terminal.rx[6]=='B') kanal=1;
        else if(Terminal.rx[6]=='C') kanal=2;
        else if(Terminal.rx[6]=='D') kanal=3;
        else if(Terminal.rx[6]=='E') kanal=4;
        else if(Terminal.rx[6]=='F') kanal=5;
        else if(Terminal.rx[6]=='G') kanal=6;
        else if(Terminal.rx[6]=='H') kanal=7;
        else if(Terminal.rx[6]=='J') kanal=8;
        else if(Terminal.rx[6]=='K') kanal=9;
        else if(Terminal.rx[6]=='L') kanal=10;
        else mode=99;
      #else
        // Hardware = myAVR MK1/MK2-Board + mySmartControl
        if(Terminal.rx[6]=='B') kanal=0;
        else if(Terminal.rx[6]=='C') kanal=1;
        else if(Terminal.rx[6]=='D') kanal=2;
        else mode=99;
      #endif

      if((mode==2) && (Terminal.rx[7]=='=')) {
        // String umwandeln in Zahl
        int_wert=hex2int(Terminal.rx,8,2);
        if(int_wert>=0) {
          wert=(unsigned char)(int_wert);
          // DDR setzen
          #if TERMINAL_HW_MODE==0
            // Hardware = myAVR MK3-Board
            if(kanal==0) DDRA=wert;
            else if(kanal==1) DDRB=wert;
            else if(kanal==2) DDRC=wert;
            else if(kanal==3) DDRD=wert;
            else if(kanal==4) DDRE=wert;
            else if(kanal==5) DDRF=wert;
            else if(kanal==6) DDRG=wert;
            else if(kanal==7) DDRH=wert;
            else if(kanal==8) DDRJ=wert;
            else if(kanal==9) DDRK=wert;
            else if(kanal==10) DDRL=wert;
          #else
            // Hardware = myAVR MK1/MK2-Board + mySmartControl
            if(kanal==0) DDRB=wert;
            else if(kanal==1) DDRC=wert;
            else if(kanal==2) DDRD=wert;
          #endif          
          strcpy(Terminal.tx,"WR_DDR");
          Terminal.tx[6]=Terminal.rx[6];
          Terminal.tx[7]='=';
          Terminal.tx[8]=0x00;          
        }
        else {
          mode=99;
        }
      }
    }
  }
  #endif
  //----------------------------------
  // IO-PORT auslesen
  // RX = "RD_PORTx"  (x=A,B,c...L)
  // TX = "RD_PORTx=ww" (ww=wert in Hex)
  //----------------------------------
  #if USE_MK3_TERMINAL_PORT==1
  else if(strncmp(Terminal.rx,"RD_PORT",7)==0) {
    if(strlen(Terminal.rx)==8) {
      mode=1;
      #if TERMINAL_HW_MODE==0
        // Hardware = myAVR MK3-Board
        if(Terminal.rx[7]=='A') wert=PINA;
        else if(Terminal.rx[7]=='B') wert=PINB;
        else if(Terminal.rx[7]=='C') wert=PINC;
        else if(Terminal.rx[7]=='D') wert=PIND;
        else if(Terminal.rx[7]=='E') wert=PINE;
        else if(Terminal.rx[7]=='F') wert=PINF;
        else if(Terminal.rx[7]=='G') wert=PING;
        else if(Terminal.rx[7]=='H') wert=PINH;
        else if(Terminal.rx[7]=='J') wert=PINJ;
        else if(Terminal.rx[7]=='K') wert=PINK;
        else if(Terminal.rx[7]=='L') wert=PINL;
        else mode=99;
      #else
        // Hardware = myAVR MK1/MK2-Board + mySmartControl
        if(Terminal.rx[7]=='B') wert=PINB;
        else if(Terminal.rx[7]=='C') wert=PINC;
        else if(Terminal.rx[7]=='D') wert=PIND;
        else mode=99;
      #endif

      if(mode==1) {
        strcpy(Terminal.tx,"RD_PORT");
        Terminal.tx[7]=Terminal.rx[7];
        Terminal.tx[8]='=';
        Terminal.tx[9]=0x00;
      }
    }
  }
  //----------------------------------
  // IO-PORT setzen
  // RX = "WR_PORTx=ww"  (x=A,B,c...L / ww=Wert in Hex)
  // TX = "WR_PORTx=ww_OK" (ww=wert in Hex)
  //----------------------------------
  else if(strncmp(Terminal.rx,"WR_PORT",7)==0) {
    if(strlen(Terminal.rx)==11) {
      mode=2;
      #if TERMINAL_HW_MODE==0
        // Hardware = myAVR MK3-Board
        if(Terminal.rx[7]=='A') kanal=0;
        else if(Terminal.rx[7]=='B') kanal=1;
        else if(Terminal.rx[7]=='C') kanal=2;
        else if(Terminal.rx[7]=='D') kanal=3;
        else if(Terminal.rx[7]=='E') kanal=4;
        else if(Terminal.rx[7]=='F') kanal=5;
        else if(Terminal.rx[7]=='G') kanal=6;
        else if(Terminal.rx[7]=='H') kanal=7;
        else if(Terminal.rx[7]=='J') kanal=8;
        else if(Terminal.rx[7]=='K') kanal=9;
        else if(Terminal.rx[7]=='L') kanal=10;
        else mode=99;
      #else
        // Hardware = myAVR MK1/MK2-Board + mySmartControl
        if(Terminal.rx[7]=='B') kanal=0;
        else if(Terminal.rx[7]=='C') kanal=1;
        else if(Terminal.rx[7]=='D') kanal=2;
        else mode=99;
      #endif

      if((mode==2) && (Terminal.rx[8]=='=')) {
        // String umwandeln in Zahl
        int_wert=hex2int(Terminal.rx,9,2);
        if(int_wert>=0) {
          wert=(unsigned char)(int_wert);
          // wert am Port ausgeben
          #if TERMINAL_HW_MODE==0
            // Hardware = myAVR MK3-Board
            if(kanal==0) PORTA=wert;
            else if(kanal==1) PORTB=wert;
            else if(kanal==2) PORTC=wert;
            else if(kanal==3) PORTD=wert;
            else if(kanal==4) PORTE=wert;
            else if(kanal==5) PORTF=wert;
            else if(kanal==6) PORTG=wert;
            else if(kanal==7) PORTH=wert;
            else if(kanal==8) PORTJ=wert;
            else if(kanal==9) PORTK=wert;
            else if(kanal==10) PORTL=wert;
          #else
            // Hardware = myAVR MK1/MK2-Board + mySmartControl
            if(kanal==0) PORTB=wert;
            else if(kanal==1) PORTC=wert;
            else if(kanal==2) PORTD=wert;
          #endif
          
          strcpy(Terminal.tx,"WR_PORT");
          Terminal.tx[7]=Terminal.rx[7];
          Terminal.tx[8]='=';
          Terminal.tx[9]=0x00;          
        }
        else {
          mode=99;
        }
      }
    }
  } 
  #endif
  //----------------------------------
  // IO-BIT auslesen
  // RX = "RD_BITxy"  (x=A,B,c...L / y=0...7)
  // TX = "RD_BITxy=w" (w=wert in Hex)
  //----------------------------------
  #if USE_MK3_TERMINAL_BIT==1
  else if(strncmp(Terminal.rx,"RD_BIT",6)==0) {
    if(strlen(Terminal.rx)==8) {
      mode=3;
      #if TERMINAL_HW_MODE==0
        // Hardware = myAVR MK3-Board
        if(Terminal.rx[6]=='A') wert=PINA;
        else if(Terminal.rx[6]=='B') wert=PINB;
        else if(Terminal.rx[6]=='C') wert=PINC;
        else if(Terminal.rx[6]=='D') wert=PIND;
        else if(Terminal.rx[6]=='E') wert=PINE;
        else if(Terminal.rx[6]=='F') wert=PINF;
        else if(Terminal.rx[6]=='G') wert=PING;
        else if(Terminal.rx[6]=='H') wert=PINH;
        else if(Terminal.rx[6]=='J') wert=PINJ;
        else if(Terminal.rx[6]=='K') wert=PINK;
        else if(Terminal.rx[6]=='L') wert=PINL;
        else mode=99;
      #else
        // Hardware = myAVR MK1/MK2-Board + mySmartControl
        if(Terminal.rx[6]=='B') wert=PINB;
        else if(Terminal.rx[6]=='C') wert=PINC;
        else if(Terminal.rx[6]=='D') wert=PIND;
        else mode=99;
      #endif

      if(mode==3) {
        if(Terminal.rx[7]=='0') wert&=0x01;
        else if(Terminal.rx[7]=='1') wert&=0x02;
        else if(Terminal.rx[7]=='2') wert&=0x04;
        else if(Terminal.rx[7]=='3') wert&=0x08;
        else if(Terminal.rx[7]=='4') wert&=0x10;
        else if(Terminal.rx[7]=='5') wert&=0x20;
        else if(Terminal.rx[7]=='6') wert&=0x40;
        else if(Terminal.rx[7]=='7') wert&=0x80;
        else mode=99;

        if(mode==3) {
          strcpy(Terminal.tx,"RD_BIT");
          Terminal.tx[6]=Terminal.rx[6];
          Terminal.tx[7]=Terminal.rx[7];
          Terminal.tx[8]='=';
          if(wert==0) Terminal.tx[9]='0';
          else Terminal.tx[9]='1';
          Terminal.tx[10]=0x00;
        }
      }
    }
  }
  //----------------------------------
  // IO-BIT setzen
  // RX = "WR_BITxy=w"  (x=A,B,c...L / y=0...7  / w=0,1)
  // TX = "WR_BITxy=w_OK" (w=wert in Hex)
  //----------------------------------
  else if(strncmp(Terminal.rx,"WR_BIT",6)==0) {
    if(strlen(Terminal.rx)==10) {
      mode=3;
      #if TERMINAL_HW_MODE==0
        // Hardware = myAVR MK3-Board
        if(Terminal.rx[6]=='A') kanal=0;
        else if(Terminal.rx[6]=='B') kanal=1;
        else if(Terminal.rx[6]=='C') kanal=2;
        else if(Terminal.rx[6]=='D') kanal=3;
        else if(Terminal.rx[6]=='E') kanal=4;
        else if(Terminal.rx[6]=='F') kanal=5;
        else if(Terminal.rx[6]=='G') kanal=6;
        else if(Terminal.rx[6]=='H') kanal=7;
        else if(Terminal.rx[6]=='J') kanal=8;
        else if(Terminal.rx[6]=='K') kanal=9;
        else if(Terminal.rx[6]=='L') kanal=10;
        else mode=99;
      #else
        // Hardware = myAVR MK1/MK2-Board + mySmartControl
        if(Terminal.rx[6]=='B') kanal=0;
        else if(Terminal.rx[6]=='C') kanal=1;
        else if(Terminal.rx[6]=='D') kanal=2;
        else mode=99;
      #endif		 

      if(mode==3) {
        if(Terminal.rx[7]=='0') wert=0x01;
        else if(Terminal.rx[7]=='1') wert=0x02;
        else if(Terminal.rx[7]=='2') wert=0x04;
        else if(Terminal.rx[7]=='3') wert=0x08;
        else if(Terminal.rx[7]=='4') wert=0x10;
        else if(Terminal.rx[7]=='5') wert=0x20;
        else if(Terminal.rx[7]=='6') wert=0x40;
        else if(Terminal.rx[7]=='7') wert=0x80;
        else mode=99;

        if((mode==3) && (Terminal.rx[8]=='=')) {
          if(Terminal.rx[9]=='0') {
            // Pin auf Lo
            #if TERMINAL_HW_MODE==0
              // Hardware = myAVR MK3-Board
              if(kanal==0) PORTA&=~wert;
              else if(kanal==1) PORTB&=~wert;
              else if(kanal==2) PORTC&=~wert;
              else if(kanal==3) PORTD&=~wert;
              else if(kanal==4) PORTE&=~wert;
              else if(kanal==5) PORTF&=~wert;
              else if(kanal==6) PORTG&=~wert;
              else if(kanal==7) PORTH&=~wert;
              else if(kanal==8) PORTJ&=~wert;
              else if(kanal==9) PORTK&=~wert;
              else if(kanal==10) PORTL&=~wert;
            #else
              // Hardware = myAVR MK1/MK2-Board + mySmartControl
              if(kanal==0) PORTB&=~wert;
              else if(kanal==1) PORTC&=~wert;
              else if(kanal==2) PORTD&=~wert;
            #endif
          }
          else if(Terminal.rx[9]=='1') {
            // Pin auf Hi
            #if TERMINAL_HW_MODE==0
              // Hardware = myAVR MK3-Board
              if(kanal==0) PORTA|=wert;
              else if(kanal==1) PORTB|=wert;
              else if(kanal==2) PORTC|=wert;
              else if(kanal==3) PORTD|=wert;
              else if(kanal==4) PORTE|=wert;
              else if(kanal==5) PORTF|=wert;
              else if(kanal==6) PORTG|=wert;
              else if(kanal==7) PORTH|=wert;
              else if(kanal==8) PORTJ|=wert;
              else if(kanal==9) PORTK|=wert;
              else if(kanal==10) PORTL|=wert;
            #else
              // Hardware = myAVR MK1/MK2-Board + mySmartControl
              if(kanal==0) PORTB|=wert;
              else if(kanal==1) PORTC|=wert;
              else if(kanal==2) PORTD|=wert;
            #endif
          }
          else {
            mode=99;
          }

          if(mode==3) {
            strcpy(Terminal.tx,"WR_BIT");
            Terminal.tx[6]=Terminal.rx[6];
            Terminal.tx[7]=Terminal.rx[7];
            Terminal.tx[8]='=';
            Terminal.tx[9]=Terminal.rx[9];
            Terminal.tx[10]=0x00;
            strcat(Terminal.tx,"_OK");
          }
        }
      }     
    }
  }
  #endif
  //----------------------------------
  // ADC auslesen
  // RX = "RD_ADCx"  (x=0...7)
  // TX = "RD_ADCx=www_OK" (www=wert in Hex)
  //----------------------------------
  #if USE_MK3_TERMINAL_ADC==1
  else if(strncmp(Terminal.rx,"RD_ADC",6)==0) {
    if(strlen(Terminal.rx)==7) {
      mode=4;
      if(Terminal.rx[6]=='0') uint_wert=MK3_ADC_READ_CH(ADC0);
      else if(Terminal.rx[6]=='1') uint_wert=MK3_ADC_READ_CH(ADC1);
      else if(Terminal.rx[6]=='2') uint_wert=MK3_ADC_READ_CH(ADC2);
      else if(Terminal.rx[6]=='3') uint_wert=MK3_ADC_READ_CH(ADC3);
      else if(Terminal.rx[6]=='4') uint_wert=MK3_ADC_READ_CH(ADC4);
      else if(Terminal.rx[6]=='5') uint_wert=MK3_ADC_READ_CH(ADC5);
      else if(Terminal.rx[6]=='6') uint_wert=MK3_ADC_READ_CH(ADC6);
      else if(Terminal.rx[6]=='7') uint_wert=MK3_ADC_READ_CH(ADC7);
      else mode=99;

      if(mode==4) {
        strcpy(Terminal.tx,"RD_ADC");
        Terminal.tx[6]=Terminal.rx[6];
        Terminal.tx[7]='=';
        Terminal.tx[8]=0x00;
      }
    }
  }
  #endif  

  //----------------------------------
  // antwort-telegramm zusammenbauen
  //----------------------------------
  if(mode==0) {
    // keine Antwort notwendig
  }
  else if(mode==1) {
    // fuer 2 stellig Hex lesen
    sprintf(buf,"%02X",wert);
    strcat(Terminal.tx,buf);
    MK3_V24_OUT(Terminal.com,Terminal.tx,RX_END_CRLF);
  }
  else if(mode==2) {
    // fuer 2 stellig Hex schreiben
    sprintf(buf,"%02X",wert);
    strcat(Terminal.tx,buf);
    strcat(Terminal.tx,"_OK");
    MK3_V24_OUT(Terminal.com,Terminal.tx,RX_END_CRLF);
  }
  else if(mode==3) {
    // fuer direkten Text
    MK3_V24_OUT(Terminal.com,Terminal.tx,RX_END_CRLF);
  }
  else if(mode==4) {
    // fuer 3 stellig Hex lesen
    sprintf(buf,"%03X",uint_wert);
    strcat(Terminal.tx,buf);
    MK3_V24_OUT(Terminal.com,Terminal.tx,RX_END_CRLF);
  }
  else if(mode==99) {
    // Kommando unbekannt
    MK3_V24_OUT(Terminal.com,"CMD=?",RX_END_CRLF);
    ret_wert=CMD_NC;
  }

  return(ret_wert);
}

//-----------------------------------------------------------------------------
// wandeln eines Teil-Strings in einen integer-wert
// erlaubte zahl von 000h=(0d) bis FFFh=(4095d)
// falls zahl nicht erlaubt-> rueckgabe = -1
//-----------------------------------------------------------------------------
int hex2int(char *ptr,unsigned int start,char laenge) {
  int ret_wert=-1;
  int z2,z1,z0;

  if(laenge==1) {
    // eine einzelne Ziffer (0...F)
    z0=hex2ziffer(ptr[start]);
    if(z0>=0) {
      ret_wert=z0;
    }
  }
  else if(laenge==2) {
    // zwei Ziffern (00...FF)
    z1=hex2ziffer(ptr[start]);
    z0=hex2ziffer(ptr[start+1]);
    if((z1>=0) && (z0>=0)) {
      ret_wert=(z1*16)+z0;
    }
  }
  else if(laenge==3) {
    // drei Ziffern (000...FFF)
    z2=hex2ziffer(ptr[start]);
    z1=hex2ziffer(ptr[start+1]);
    z0=hex2ziffer(ptr[start+2]);
    if((z2>=0) && (z1>=0) && (z0>=0)) {
      ret_wert=(z2*256)+(z1*16)+z0;
    }
  }

  return(ret_wert);
}


//-----------------------------------------------------------------------------
// wandelt ein Zeichen in einen integer-wert
// erlaubtes zeichen = 0...9 , a...f, A...F
// falls zahl nicht erlaubt-> rueckgabe = -1
//----------------------------------------------------------------------------- 
int hex2ziffer(char ziffer) {
  int ret_wert=-1;

  if((ziffer>='0') && (ziffer<='9')) {
    ret_wert=ziffer-'0';
  }
  else if((ziffer>='a') && (ziffer<='f')) {
    ret_wert=ziffer-'a'+10;
  }
  else if((ziffer>='A') && (ziffer<='F')) {
    ret_wert=ziffer-'A'+10;
  }

  return(ret_wert);
}










