//-----------------------------------------------------------------------------
// MK3_2560_V24
// Version 1.6
// 19.08.2012 / UB
// "www.Mikrocontroller-4U.de"
// mc-4u(@)t-online.de
//-----------------------------------------------------------------------------
// Fuer das myAVR MK3 Board (oder MK2 Board)
// mit ATMega-2560 @ 16MHz (oder ATMega8) 
// RS232 Ein-/Ausgabe
//-----------------------------------------------------------------------------
// das File muss im Ordner
// "myAVR/WinAVR/AVR/Include/UB_MK3_2560/"
// liegen und mit #include <UB_MK3_2560/MK3_2560_V24.h>
// eingebunden werden
//-----------------------------------------------------------------------------
// Belegt : @MK3 (ATMega2560)
//          bei COM0 = PortE Bit0 (RxD) + Bit1 (TxD)
//          bei COM1 = PortD Bit2 (RxD) + Bit3 (TxD) >> USB auf mySmartUSB
//          bei COM2 = PortH Bit0 (RxD) + Bit1 (TxD)
//          bei COM3 = PortJ Bit0 (RxD) + Bit1 (TxD) >> MINI-USB auf myAVRStamp
// Belegt : @MK2 (ATMega8)
//          bei COM0 = PortD Bit0 (RxD) + Bit1 (TxD) >> USB auf mySmartUSB
//
// Hinweis :
//
// > die DEFINE-Schalter "V24_HW_MODE" und "V24_QUARZ_TYP"
//   muessen zur Hardware passen !!
//
// Com-Port und Baudrate kann eingestellt werden (bei MEGA8 -> nur COM0)
// Frame ist fest eingestellt : 1 Startbit, 8 Datenbits, keine Paritaet, 1 Stopbit
// 
// Senden : sendet einen String und sendet zum Abschluss (RX_END)
// Empfangen : empfaengt im Interrupt und fuellt den Puffer,
//             bis ein 0x0D empfangen wird, dann meldung das ein String
//             im Puffer zum abholen liegt
//-----------------------------------------------------------------------------


#include <avr/interrupt.h>


//-----------------------------------------------------------------------------
// Hier koennen Funktionen die nicht benutzt werden
// abgeschaltet werden...einfach den DEFINE von "1" auf "0" aendern
//-----------------------------------------------------------------------------

#define USE_MK3_V24_OUT 1
#define USE_MK3_V24_IN 1

//-----------------------------------------------------------------------------
// Hardware auswahl
//-----------------------------------------------------------------------------
#define V24_HW_MODE    0       // 0=MK3-Board, 1=MK1/MK2-Board/mySmartControl
#define V24_QUARZ_TYP  0       // 0=16MHz, 1=8MHz, 2=3.6864MHz



//-----------------------------------------------------------------------------
// INFOS zur Benutzung :
// 01. Funktion "MK3_INIT_V24" starten (fuer jede Schnittstelle einzeln)
// 02. Funktion "MK3_V24_OUT" sendet einen String an den ComPort
// 03. Funktion "MK3_V24_IN" starten (fuer einen ComPort)
// 04. Wenn Rueckgabe von 03.=TRUE dann wurde der empfangene String uebergeben 
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Defines
//-----------------------------------------------------------------------------

#define RX_PUFFER_LEN 100    // laenge des Empfangs-Puffers


//-----------------------------------------------------------------------------
// Enumerationen fuer die Funktioen
//-----------------------------------------------------------------------------


typedef enum {
  COM0 = 0,
  COM1,
  COM2,
  COM3
}COMPORT;

typedef enum {
  B4800 = 0,
  B9600,
  B14400,
  B19200,
  B38400,
  B57600,
  B115200,
  B230400
}BAUDRATE;

typedef enum {
  RX_END_NOP = 0,  // sendet am String-Ende (nichts)
  RX_END_CR,       // sendet am String-Ende (0x0D)
  RX_END_LF,       // sendet am String-Ende (0x0A)
  RX_END_CRLF      // sendet am String-Ende (0x0D,0x0A)
}RX_END;



//-----------------------------------------------------------------------------
// Funktionen die vom MAIN benutzt werden koennen
//-----------------------------------------------------------------------------
void MK3_INIT_V24(COMPORT port, BAUDRATE baud);	        // zum initialisieren des V24-Port
void MK3_V24_OUT(COMPORT port, const char *ptr,RX_END ende);  // sendet String per V24
bool MK3_V24_IN(COMPORT port,char *ptr);                // String vom Puffer auslesen
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// interne Funktionen
//-----------------------------------------------------------------------------
void v24SendAsc(COMPORT port, char wert);       // sendet ein Ascii-Zeichen per V24
void MK3_V24_READ(COMPORT port,char *ptr);      // puffer auslesen



//-----------------------------------------------------------------------------
// Defines
//-----------------------------------------------------------------------------
#define V24_CHAR_CR 0x0d	// Zeichen fuer CR (am Ende eines Strings)
#define V24_CHAR_LF 0x0a	// Zeichen fuer LF (am Ende eines Strings)
#define V24_ASCII_MIN 0x20      // kleinstes Ascii-Zeichen
#define V24_ASCII_MAX 0x7e      // groesstes Ascii-Zeichen


#if USE_MK3_V24_IN==1
char rx0_buf[RX_PUFFER_LEN];
unsigned int rx0_pos=0;
bool rx0_ok=false;
#if V24_HW_MODE==0
char rx1_buf[RX_PUFFER_LEN];
char rx2_buf[RX_PUFFER_LEN];
char rx3_buf[RX_PUFFER_LEN];
unsigned int rx1_pos=0;
unsigned int rx2_pos=0;
unsigned int rx3_pos=0;
bool rx1_ok=false;
bool rx2_ok=false;
bool rx3_ok=false;
#endif
#endif

//-----------------------------------------------------------------------------
void MK3_INIT_V24(COMPORT port, BAUDRATE baud) {	// zum initialisieren eines V24-Port
  unsigned int m=0;

  #if V24_QUARZ_TYP==0
    // 16MHz
    if(baud==B4800) m=207;
    if(baud==B9600) m=103;
    if(baud==B14400) m=68;
    if(baud==B19200) m=51;
    if(baud==B38400) m=25;
    if(baud==B57600) m=16;
    if(baud==B115200) m=8;
    if(baud==B230400) m=3;
  #elif V24_QUARZ_TYP==1
    // 8MHz
    if(baud==B4800) m=103;
    if(baud==B9600) m=51;
    if(baud==B14400) m=34;
    if(baud==B19200) m=25;
    if(baud==B38400) m=12;
    if(baud==B57600) m=8;
    if(baud==B115200) m=3;
    if(baud==B230400) m=1;
  #elif V24_QUARZ_TYP==2
    // 3.6864MHz
    if(baud==B4800) m=47;
    if(baud==B9600) m=23;
    if(baud==B14400) m=15;
    if(baud==B19200) m=11;
    if(baud==B38400) m=5;
    if(baud==B57600) m=3;
    if(baud==B115200) m=1;
    if(baud==B230400) m=0;
  #endif


  #if V24_HW_MODE==0
  // Hardware = myAVR MK3-Board
  if(port==COM0) {
    // baudrate einstellen
    UBRR0H=0x00;
    UBRR0L=m;
    //---------------------------
    UCSR0A=0x00;    // U2X auf LO
    //---------------------------
    UCSR0B=0x00;
    UCSR0B|=0x80;   // Rx Interrupt enable
    UCSR0B|=0x10;   // Rx enable
    UCSR0B|=0x08;   // Tx enable
    //---------------------------
    UCSR0C=0x00;    // asyncron, no parity, 1stopbit
    UCSR0C|=0x04;   // 8datenbits
    UCSR0C|=0x02;   // 8datenbits
    //---------------------------
    #if USE_MK3_V24_IN==1
    rx0_pos=0;
    rx0_buf[0]=0x00;
    rx0_ok=false;
    #endif
  }
  else if(port==COM1) {
    // baudrate einstellen
    UBRR1H=0x00;
    UBRR1L=m;
    //---------------------------
    UCSR1A=0x00;    // U2X auf LO
    //---------------------------
    UCSR1B=0x00;
    UCSR1B|=0x80;   // Rx Interrupt enable
    UCSR1B|=0x10;   // Rx enable
    UCSR1B|=0x08;   // Tx enable
    //---------------------------
    UCSR1C=0x00;    // asyncron, no parity, 1stopbit
    UCSR1C|=0x04;   // 8datenbits
    UCSR1C|=0x02;   // 8datenbits
    //---------------------------
    #if USE_MK3_V24_IN==1
    rx1_pos=0;
    rx1_buf[0]=0x00;
    rx1_ok=false;
    #endif
  }
  else if(port==COM2) {
    // baudrate einstellen
    UBRR2H=0x00;
    UBRR2L=m;
    //---------------------------
    UCSR2A=0x00;    // U2X auf LO
    //---------------------------
    UCSR2B=0x00;
    UCSR2B|=0x80;   // Rx Interrupt enable
    UCSR2B|=0x10;   // Rx enable
    UCSR2B|=0x08;   // Tx enable
    //---------------------------
    UCSR2C=0x00;    // asyncron, no parity, 1stopbit
    UCSR2C|=0x04;   // 8datenbits
    UCSR2C|=0x02;   // 8datenbits
    //---------------------------
    #if USE_MK3_V24_IN==1
    rx2_pos=0;
    rx2_buf[0]=0x00;
    rx2_ok=false;
    #endif
  }
  else if(port==COM3) {
    // baudrate einstellen
    UBRR3H=0x00;
    UBRR3L=m;
    //---------------------------
    UCSR3A=0x00;    // U2X auf LO
    //---------------------------
    UCSR3B=0x00;
    UCSR3B|=0x80;   // Rx Interrupt enable
    UCSR3B|=0x10;   // Rx enable
    UCSR3B|=0x08;   // Tx enable
    //---------------------------
    UCSR3C=0x00;    // asyncron, no parity, 1stopbit
    UCSR3C|=0x04;   // 8datenbits
    UCSR3C|=0x02;   // 8datenbits
    //---------------------------
    #if USE_MK3_V24_IN==1
    rx3_pos=0;
    rx3_buf[0]=0x00;
    rx3_ok=false;
    #endif
  }
  #else
  // Hardware = myAVR MK1/MK2-Board + mySmartControl 
    // baudrate einstellen
    UBRRH=0x00;
    UBRRL=m;
    //---------------------------
    UCSRA=0x00;     // U2X auf LO
    //---------------------------
    UCSRB=0x00;
    UCSRB|=0x80;    // Rx Interrupt enable
    UCSRB|=0x10;    // Rx enable
    UCSRB|=0x08;    // Tx enable
    //---------------------------
    UCSRC=0x00;     // asyncron, no parity, 1stopbit
    UCSRC = 0x80 | 0x06; // 8datenbits
    //---------------------------
    #if USE_MK3_V24_IN==1
    rx0_pos=0;
    rx0_buf[0]=0x00;
    rx0_ok=false;
    #endif
  #endif

  SREG|=0x80; // Alle Interrupts einschalten

}

//-----------------------------------------------------------------------------
#if USE_MK3_V24_OUT==1
void MK3_V24_OUT(COMPORT port, const char *ptr,RX_END ende) { // sendet String per V24
  unsigned int n;
  char c;

  for(n=0;n<=255;n++) {
    c=ptr[n];
    if(c==0) {
      if(ende==RX_END_CR) {
        v24SendAsc(port, V24_CHAR_CR); // am ende ein Return-Zeichen
      }
      else if(ende==RX_END_LF) {
        v24SendAsc(port, V24_CHAR_LF); // am ende ein LineFeed-Zeichen
      }
      else if(ende==RX_END_CRLF) {
        v24SendAsc(port, V24_CHAR_CR); // am ende ein Return-Zeichen
        v24SendAsc(port, V24_CHAR_LF); // am ende ein LineFeed-Zeichen
      }
      break; 
    }
    else {
      v24SendAsc(port, c);
    }   
  }
}
#endif

//-----------------------------------------------------------------------------
#if USE_MK3_V24_IN==1
bool MK3_V24_IN(COMPORT port,char *ptr) {        // String vom Puffer auslesen
  bool ret_wert=false;

  #if V24_HW_MODE==0
    // Hardware = myAVR MK3-Board
    if(port==COM0) {
      ret_wert=rx0_ok;
    }
    else if(port==COM1) {
      ret_wert=rx1_ok;
    }
    else if(port==COM2) {
      ret_wert=rx2_ok;
    }
    else if(port==COM3) {
      ret_wert=rx3_ok;
    }
  #else
  // Hardware = myAVR MK1/MK2-Board + mySmartControl    
    ret_wert=rx0_ok;
  #endif

  if(ret_wert==true) {
    // wenn etwas im puffer steht
    MK3_V24_READ(port,ptr);
  }

  return ret_wert;
}
#endif

//-----------------------------------------------------------------------------
#if USE_MK3_V24_IN==1
void MK3_V24_READ(COMPORT port,char *ptr) {      // puffer auslesen
  unsigned int n;
  char c;

  #if V24_HW_MODE==0
    // Hardware = myAVR MK3-Board
    if(port==COM0) {
      for(n=0;n<=RX_PUFFER_LEN;n++) {
        c=rx0_buf[n];
        ptr[n]=c;
        if(c==0) {
          break; 
        }
      }
      rx0_pos=0;
      rx0_buf[0]=0x00;
      rx0_ok=false;
    }
    else if(port==COM1) {
      for(n=0;n<=RX_PUFFER_LEN;n++) {
        c=rx1_buf[n];
        ptr[n]=c;
        if(c==0) {
          break; 
        }
      }
      rx1_pos=0;
      rx1_buf[0]=0x00;
      rx1_ok=false;   
    }
    else if(port==COM2) {
      for(n=0;n<=RX_PUFFER_LEN;n++) {
        c=rx2_buf[n];
        ptr[n]=c;
        if(c==0) {
          break; 
        }
      }
      rx2_pos=0;
      rx2_buf[0]=0x00;
      rx2_ok=false;
    }
    else if(port==COM3) {
      for(n=0;n<=RX_PUFFER_LEN;n++) {
        c=rx3_buf[n];
        ptr[n]=c;
        if(c==0) {
          break; 
        }
      }
      rx3_pos=0;
      rx3_buf[0]=0x00;
      rx3_ok=false;
    }
  #else
  // Hardware = myAVR MK1/MK2-Board + mySmartControl    
    for(n=0;n<=RX_PUFFER_LEN;n++) {
      c=rx0_buf[n];
      ptr[n]=c;
      if(c==0) {
        break; 
      }
    }
    rx0_pos=0;
    rx0_buf[0]=0x00;
    rx0_ok=false;
  #endif
}
#endif

//-----------------------------------------------------------------------------
#if USE_MK3_V24_OUT==1
void v24SendAsc(COMPORT port, char wert) {      // sendet ein Ascii-Zeichen per V24
  #if V24_HW_MODE==0
    // Hardware = myAVR MK3-Board
    if(port==COM0) {
      while(!(UCSR0A & 0x20)); // warte bis sende puffer leer
      UDR0=wert;
    }
    else if(port==COM1) {
      while(!(UCSR1A & 0x20)); // warte bis sende puffer leer
      UDR1=wert;
    }
    else if(port==COM2) {
      while(!(UCSR2A & 0x20)); // warte bis sende puffer leer
      UDR2=wert;
    }
    else if(port==COM3) {
      while(!(UCSR3A & 0x20)); // warte bis sende puffer leer
      UDR3=wert;
    }
  #else
  // Hardware = myAVR MK1/MK2-Board + mySmartControl
    while(!(UCSRA & 0x20)); // warte bis sende puffer leer
    UDR=wert;
  #endif
}
#endif

#if V24_HW_MODE==0
  // Hardware = myAVR MK3-Board
//-----------------------------------------------------------------------------
ISR (USART0_RX_vect) {                          // ISR : Com0 Rx-Complete
  #if USE_MK3_V24_IN==1
  char wert;

  wert=UDR0; // daten abholen
  if(wert==V24_CHAR_CR) {
    rx0_ok=true;
  }
  else {
    if(rx0_ok==false) {
      if(rx0_pos<RX_PUFFER_LEN) {
        if((wert>=V24_ASCII_MIN) && (wert<=V24_ASCII_MAX)) {
          rx0_buf[rx0_pos]=wert;
          rx0_buf[rx0_pos+1]=0x00;
          rx0_pos++;
        }
      }
    }
  }  
  #endif
}

//-----------------------------------------------------------------------------
ISR (USART1_RX_vect) {                          // ISR : Com1 Rx-Complete
  #if USE_MK3_V24_IN==1
  char wert;

  wert=UDR1; // daten abholen
  if(wert==V24_CHAR_CR) {
    rx1_ok=true;
  }
  else {
    if(rx1_ok==false) {
      if(rx1_pos<RX_PUFFER_LEN) {
        if((wert>=V24_ASCII_MIN) && (wert<=V24_ASCII_MAX)) {
          rx1_buf[rx1_pos]=wert;
          rx1_buf[rx1_pos+1]=0x00;
          rx1_pos++;
        }
      }
    }
  }  
  #endif
}

//-----------------------------------------------------------------------------
ISR (USART2_RX_vect) {                          // ISR : Com2 Rx-Complete
  #if USE_MK3_V24_IN==1
  char wert;

  wert=UDR2; // daten abholen
  if(wert==V24_CHAR_CR) {
    rx2_ok=true;
  }
  else {
    if(rx2_ok==false) {
      if(rx2_pos<RX_PUFFER_LEN) {
        if((wert>=V24_ASCII_MIN) && (wert<=V24_ASCII_MAX)) {
          rx2_buf[rx2_pos]=wert;
          rx2_buf[rx2_pos+1]=0x00;
          rx2_pos++;
        }
      }
    }
  }  
  #endif
}

//-----------------------------------------------------------------------------
ISR (USART3_RX_vect) {                          // ISR : Com3 Rx-Complete
  #if USE_MK3_V24_IN==1
  char wert;

  wert=UDR3; // daten abholen
  if(wert==V24_CHAR_CR) {
    rx3_ok=true;
  }
  else {
    if(rx3_ok==false) {
      if(rx3_pos<RX_PUFFER_LEN) {
        if((wert>=V24_ASCII_MIN) && (wert<=V24_ASCII_MAX)) {
          rx3_buf[rx3_pos]=wert;
          rx3_buf[rx3_pos+1]=0x00;
          rx3_pos++;
        }
      }
    }
  }  
  #endif
}

#else
  // Hardware = myAVR MK1/MK2-Board + mySmartControl
//-----------------------------------------------------------------------------
ISR (USART_RXC_vect) {                          // ISR : Com Rx-Complete
  #if USE_MK3_V24_IN==1
  char wert;

  wert=UDR; // daten abholen
  if(wert==V24_CHAR_CR) {
    rx0_ok=true;
  }
  else {
    if(rx0_ok==false) {
      if(rx0_pos<RX_PUFFER_LEN) {
        if((wert>=V24_ASCII_MIN) && (wert<=V24_ASCII_MAX)) {
          rx0_buf[rx0_pos]=wert;
          rx0_buf[rx0_pos+1]=0x00;
          rx0_pos++;
        }
      }
    }
  }  
  #endif
}
#endif



